/******************************************************************************
Author:        David Szybala
Website:       www.davidszybala.com
Alt Website:   www.twistedgerbil.com
Program:       Delete_Empty_Layers.ms
Date:          10/25/16
Description:   This script will delete all empty layers in the current scene.
               WARNING: Deleting empty layers must be handled differently
               depending on the max version!
******************************************************************************/


( --local scope
    global empty_layer_rollout
    
    /**************************************************************************
    Returns a list of root layer names (for use with max 2016+)
    **************************************************************************/
    fn get_root_layer_names =
    (
        local layer_names = #()
        local current_layer = undefined
        
        for i = 1 to (layerManager.count - 1) do
        (
            current_layer = layerManager.getLayer i
            
            if current_layer.getParent() == undefined do
                append layer_names current_layer.name
        ) --end for
        
        layer_names --return value
    ) --end fn
    
    /**************************************************************************
    Returns a 'path' name of a layer (eg Root\Parent1\Parent2\Layer)
    **************************************************************************/
    fn get_path_name layer_node =
    (
        local layer_path = layer_node.name
        
        while layer_node.getParent() != undefined do
        (
            layer_node = layer_node.getParent()
            layer_path = layer_node.name + "\\" + layer_path
        ) --end while

        layer_path --return value
    ) --end fn
    
    /**************************************************************************
    Sorts a list by path name (for use with max 2015)
    **************************************************************************/
    fn sort_by_path_name item_one item_two =
	(
        local name_one = ""
        local name_two = ""

        path_one = get_path_name item_one
        path_two = get_path_name item_two

		case of
		(
			(path_one < path_two): -1
			(path_one > path_two): 1
			default: 0
		) --end case
	) --end fn
    
    /**************************************************************************
    Returns a list of empty layer names (for use with max 2014 or lower)
    **************************************************************************/
    fn get_empty_layer_names =
    (
        local layer_names = #()
        local current_layer = undefined
        
        for i = 1 to (layerManager.count - 1) do
        (
            current_layer = layerManager.getLayer i
            current_layer.nodes &layer_nodes
            
            if layer_nodes.count == 0 do
                append layer_names current_layer.name
        ) --end for
        
        layer_names --return value
    ) --end fn
    
    /**************************************************************************
    Delete empty layer for max 2016+
    **************************************************************************/
    fn delete_empty_layers_2016 =
    (
        local layer_names = get_root_layer_names()
        local total_layers = layerManager.count
        local percent = 0
        
        for i = 1 to layer_names.count do
        (
            layerManager.deleteLayerHierarchy layer_names[i] forceDelete:false
            percent = 100.0 * i / layer_names.count
            empty_layer_rollout.prog_bar.value = percent
            if ((maxVersion())[1] / 1000) > 12 do windows.processPostedMessages()
        ) --end for
        
        print("INFO: " + ((total_layers - layerManager.count) as string) + " empty layers deleted")
    ) --end fn
    
    /**************************************************************************
    Delete empty layer for max 2015. Hierarchies should be preserved. NOTE:
    2015 allows nested layers but does not include the built in function to
    properly clean them
    **************************************************************************/
    fn delete_empty_layers_2015 =
    (
        local current_layers = for i = 1 to (layerManager.count - 1) collect (layerManager.getLayer i)
        local progress_counter = 1
        local counter = 0
        
        qsort current_layers sort_by_path_name
        
        for i = current_layers.count to 1 by -1 do
        (
            current_layers[i].nodes &layer_nodes
                
            if layer_nodes.count == 0 do
            (
                if current_layers[i].getNumChildren() == 0 do
                (
                    layerManager.deleteLayerByName current_layers[i].name
                    counter += 1
                ) --end if
            ) --end if
            
            percent = 100.0 * progress_counter / current_layers.count
            empty_layer_rollout.prog_bar.value = percent
            if ((maxVersion())[1] / 1000) > 12 do windows.processPostedMessages()
            progress_counter += 1
        ) --end for
        
        print("INFO: " + (counter as string) + " empty layers deleted")
    ) --end fn
    
    /**************************************************************************
    Delete empty layers for max 2014 and lower. WARNING: Deleting lots of
    layers in older versions of max can take a while. Displaying a progress
    bar for the user
    **************************************************************************/
    fn delete_empty_layers_2014 =
    (
        local layer_names = #()
        local percent = 0

        layer_names = get_empty_layer_names()
    
        for i = 1 to layer_names.count do
        (
            layerManager.deleteLayerByName layer_names[i]
            
            percent = 100.0 * i / layer_names.count
            empty_layer_rollout.prog_bar.value = percent
            if ((maxVersion())[1] / 1000) > 12 do windows.processPostedMessages()
        ) --end for
        
        print("INFO: " + (layer_names.count as string) + " empty layers deleted")
    ) --end fn

    /**************************************************************************
    Code Execution. WARNING: Deleting empty layers with the layer manager open can be EXTREMELY
    slow, so close the editor first!
    **************************************************************************/
    local ui_open = layerManager.isDialogOpen()
    
    rollout empty_layer_rollout "Delete Empty Layers"
    (
        label status_label "INFO: Deleting empty layers. Please wait..." pos:[5,5] width:230 height:15
        progressBar prog_bar pos:[5,25] width:230 height:10
    ) --end rollout
    
    layerManager.closeDialog()
    (layerManager.getLayer 0).current = true
    
    createDialog empty_layer_rollout 240 40
    empty_layer_rollout.prog_bar.color = [200,0,0]
    
    case of
    (
        (((maxVersion())[1] / 1000) > 17): delete_empty_layers_2016()
        (((maxVersion())[1] / 1000) > 16): delete_empty_layers_2015()
        default: delete_empty_layers_2014()
    ) --end case
    
    destroyDialog empty_layer_rollout
    
    if ui_open do
    (
        try (macros.run "Scene Explorer" "SELayerExplorer")
        catch
        (
            try(macros.run "Layers" "LayerManager")
            catch()
        ) --end catch
    ) --end if
) --end local scope
